package bench;

import java.util.Iterator;

import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;

import arq.cmdline.CmdGeneral;

import com.hp.hpl.jena.sparql.core.DatasetGraph;
import com.hp.hpl.jena.sparql.util.Utils;

import benchmark.testdriver.AbstractParameterPool;
import benchmark.testdriver.QueryMix;


/**
 * Abstract base for all command line tools that executes queries for the
 * benchmark.
 *
 * @author Olaf Hartig
 */
abstract public class CmdExecBase extends CmdGeneral
{
	final protected ModExecution modExecution = new ModExecution();


	// initialization

	protected CmdExecBase ( String[] args )
	{
		super( args );
		addModule( modExecution );
	}


	// implementation of the CmdArgModule interface

	protected void processModulesAndArgs ()
	{
	}


	// implementation of the CmdMain interface

	final protected void exec ()
	{
		initializeExecution();

		Graph data = getQueriedData();
		QueryMix queryMix = modExecution.getQueryMix();
		AbstractParameterPool parameterPool = modExecution.getParameterPool();

		execWorker( ModelFactory.createModelForGraph(data), queryMix, parameterPool );
	}

	protected String getCommandName ()
	{
		return Utils.className( this );
	}


	// implementation of the CmdGeneral interface

	protected String getSummary ()
	{
		return getCommandName() + " --dataset=<file> --idir=<directory> --qdir=<directory>";
	}


	// helpers

	abstract protected void execWorker( Model data, QueryMix queryMix, AbstractParameterPool parameterPool );

	protected void initializeExecution ()
	{
	}

	protected Graph getQueriedData ()
	{
		return modExecution.getDataset();
	}

}
