package bench;

import com.hp.hpl.jena.sparql.util.Utils;

import arq.cmdline.ArgDecl;
import arq.cmdline.ArgModuleGeneral;
import arq.cmdline.CmdArgModule;
import arq.cmdline.CmdGeneral;


/**
 * @author Olaf Hartig
 */
public class ModBenchmark implements ArgModuleGeneral
{
	final protected ArgDecl runsDecl = new ArgDecl( ArgDecl.HasValue, "runs" );
	final protected ArgDecl wDecl = new ArgDecl( ArgDecl.HasValue, "w" );

	static protected int dfltRuns = 50;
	static protected int dfltW = 10;

	private int runs;
	private int w;


	// implementation of the ArgModuleGeneral interface

	public void registerWith ( CmdGeneral cmdLine )
	{
		cmdLine.getUsage().startCategory( "Benchmark options" );

		cmdLine.add( runsDecl,
		             "--runs",
		             "Number of query mix runs (default: " + String.valueOf(dfltRuns) + ")") ;
		cmdLine.add( wDecl,
		             "--w",
		             "Number of query mix runs to warm up (default: " + String.valueOf(dfltW) + ")") ;
	}


	// implementation of the ArgModule interface

	public void processArgs ( CmdArgModule cmdLine )
	{
		if ( cmdLine.contains(runsDecl) )
		{
			try {
				runs = Integer.parseInt( cmdLine.getValue(runsDecl) );
			} catch ( NumberFormatException e ) {
				cmdLine.cmdError( "Reading the given runs failed: " + Utils.className(e) );
			}
		}
		else {
			runs = dfltRuns;
		}

		if ( cmdLine.contains(wDecl) )
		{
			try {
				w = Integer.parseInt( cmdLine.getValue(wDecl) );
			} catch ( NumberFormatException e ) {
				cmdLine.cmdError( "Reading the given runs failed: " + Utils.className(e) );
			}
		}
		else {
			w = dfltW;
		}
	}


	// accessors

	public int getRuns ()
	{
		return runs;
	}

	public int getW ()
	{
		return w;
	}


}