package bench;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Iterator;
import java.util.Vector;

import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.graph.GraphUtil;
import com.hp.hpl.jena.graph.Triple;

import com.hp.hpl.jena.sparql.core.DatasetGraph;
import com.hp.hpl.jena.sparql.util.Utils;
import org.openjena.riot.Lang;
import org.openjena.riot.RiotLoader;

import arq.cmdline.ArgDecl;
import arq.cmdline.ArgModuleGeneral;
import arq.cmdline.CmdArgModule;
import arq.cmdline.CmdGeneral;

import de.fuberlin.wiwiss.ng4j.NamedGraph;
import de.fuberlin.wiwiss.ng4j.NamedGraphSet;
// import de.fuberlin.wiwiss.ng4j.impl.NamedGraphSetImpl;
// import de.fuberlin.wiwiss.ng4j.impl.idbased.IdBasedNamedGraphSetImpl;

import de.fuberlin.wiwiss.jenaext.IdBasedGraph;
import de.fuberlin.wiwiss.jenaext.NodeDictionary;
import de.fuberlin.wiwiss.jenaext.impl.IdBasedGraphMem;
import de.fuberlin.wiwiss.jenaext.impl.NodeDictionaryImpl;
import de.fuberlin.wiwiss.jenaext.sparql.IdBasedQueryEngine;

import org.squin.dataset.QueriedDataset;
import org.squin.dataset.jenacommon.JenaIOBasedQueriedDataset;
import org.squin.dataset.jenacommon.QueriedDatasetWrappingJenaGraph;
import org.squin.dataset.query.arq.QueryEngine;


/**
 * @author Olaf Hartig
 */
public class ModGeneral implements ArgModuleGeneral
{
	final protected ArgDecl datasetDecl = new ArgDecl( ArgDecl.HasValue, "dataset" );
	final protected ArgDecl datasetImplDecl = new ArgDecl( ArgDecl.HasValue, "datasetImpl" );

	private String datasetImpl;
	private String datasetFileName;
	private File datasetFile;
	private Graph dataset;
	private Integer param1;


	// implementation of the ArgModuleGeneral interface

	public void registerWith ( CmdGeneral cmdLine )
	{
		cmdLine.getUsage().startCategory( "Benchmark data" );
		cmdLine.add( datasetDecl,
		             "--dataset",
		             "TriG file containing the benchmark dataset as generated by BSBM (mandatory)") ;
		cmdLine.add( datasetImplDecl,
		             "--datasetImpl",
		             "Specifies the implementation of the queried dataset that has to be used -- options: NG4J, MultipleIndices, IndividuallyIndexed, SingleIndex, QuadIndex, ARQ (mandatory)") ;
	}


	// implementation of the ArgModule interface

	public void processArgs ( CmdArgModule cmdLine )
	{
		dataset = null;

		if ( cmdLine.contains(datasetDecl) )
		{
			datasetFileName = cmdLine.getValue( datasetDecl );
			datasetFile = new File( datasetFileName );
			if ( ! datasetFile.exists() || ! datasetFile.canRead() ) {
				cmdLine.cmdError( "The given dataset file does not exist or is unreadable" );
			}
		}
		else {
			cmdLine.cmdError( "No dataset specified" );
		}

		if ( cmdLine.contains(datasetImplDecl) )
		{
			datasetImpl = cmdLine.getValue( datasetImplDecl );
			if ( datasetImpl.equals("MultipleIndices") )
			{
				IdBasedQueryEngine.register();
			}
			else if (    datasetImpl.equals("IndividuallyIndexed")
			          || datasetImpl.equals("SingleIndex")
			          || datasetImpl.equals("QuadIndex") )
			{
				QueryEngine.register();
			}
			else if (    ! datasetImpl.equals("ARQ")
			            && ! datasetImpl.equals("NG4J") )
			{
				cmdLine.cmdError( "The given dataset implementation is not specified correctly" );
			}
		}
		else {
			cmdLine.cmdError( "No dataset implementation specified" );
		}

		if ( cmdLine.getNumPositional() > 0 ) {
			try {
				param1 = Integer.valueOf( cmdLine.getPositionalArg(0) );
			} catch ( NumberFormatException e ) {
				cmdLine.cmdError( "Parsing the first implementation specific initialization parameter caused a " + e.getClass().getSimpleName() + ": " + e.getMessage() );
			}
		}
	}


	// accessors

	public Graph getDataset ()
	{
		if ( dataset == null ) {

			if ( datasetImpl.equals("NG4J") )
			{
				NamedGraphSet ngs = new NamedGraphSetImpl();
				readDataset( ngs, datasetFile );
				dataset = ngs.asJenaGraph( null );
			}
			else if ( datasetImpl.equals("MultipleIndices") )
			{
				NodeDictionary dict = new NodeDictionaryImpl();
				IdBasedNamedGraphSetImpl ngs = new IdBasedNamedGraphSetImpl( dict );
				readDataset( ngs, datasetFile );
				dataset = ngs.asJenaGraph( null );
			}
			else if (    datasetImpl.equals("IndividuallyIndexed")
			          || datasetImpl.equals("SingleIndex")
			          || datasetImpl.equals("QuadIndex") )
			{
				DatasetGraph dsg = RiotLoader.load( datasetFileName, Lang.TRIG );
				QueriedDataset qds;
				if ( datasetImpl.equals("IndividuallyIndexed") ) {
					qds = ( param1 == null ) ? new org.squin.dataset.hashimpl.individual.QueriedDatasetImpl() : new org.squin.dataset.hashimpl.individual.QueriedDatasetImpl( param1.intValue() );
				}
				else {
					org.squin.dataset.hashimpl.combined.CombinedIndex idx;
					if ( datasetImpl.equals("SingleIndex") ) {
						idx = ( param1 == null ) ? new org.squin.dataset.hashimpl.combined.SourceAwareTripleIndex() : new org.squin.dataset.hashimpl.combined.SourceAwareTripleIndex( param1.intValue() );
					} else {
						idx = ( param1 == null ) ? new org.squin.dataset.hashimpl.combined.QuadIndex() : new org.squin.dataset.hashimpl.combined.QuadIndex( param1.intValue() );
					}
					qds = new org.squin.dataset.hashimpl.combined.QueriedDatasetImpl( idx );
				}

long creationStartTime = System.currentTimeMillis();
				JenaIOBasedQueriedDataset ds = new JenaIOBasedQueriedDataset( dsg, qds );
long creationTime = System.currentTimeMillis() - creationStartTime;
System.out.println( "creationTime: " + String.valueOf(creationTime) );
// System.exit( 0 );
				dsg.close();
// dsg = RiotLoader.load( datasetFileName, Lang.TRIG );
// ds.putRDFGraphs( dsg );
// dsg.close();
				dataset = new QueriedDatasetWrappingJenaGraph( ds );
			}
			else if ( datasetImpl.equals("ARQ") )
			{
				DatasetGraph dsg = RiotLoader.load( datasetFileName, Lang.TRIG );
				dataset = new DatasetGraphWrappingJenaGraph( dsg );
			}
		}

		return dataset;
	}


	// helpers

	static protected void readDataset ( NamedGraphSet ngs, File datasetFile )
	{
		FileInputStream datasetStream;
		try {
			datasetStream = new FileInputStream( datasetFile );
		}
		catch ( Exception e ) {
			throw new IllegalArgumentException( "Opening the given dataset file caused a " + Utils.className(e) + ": " + e.getMessage(), e );
		}

		try {
			ngs.read( datasetStream, "TRIG", null );
		}
		catch ( Exception e ) {
			throw new IllegalArgumentException( "Reading the given dataset file caused a " + Utils.className(e) + ": " + e.getMessage(), e );
		}
	}

}