package bench;

import java.io.PrintStream;

import com.hp.hpl.jena.rdf.model.Model;

import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.sparql.resultset.ResultSetCompare;
import com.hp.hpl.jena.sparql.resultset.ResultSetMem;

import benchmark.testdriver.AbstractParameterPool;
import benchmark.testdriver.Query;
import benchmark.testdriver.QueryMix;


/**
 * Command line tool that verifies the results of benchmark queries.
 *
 * @author Olaf Hartig
 */
public class verifyResults extends CmdExecBase
{
	public static void main ( String [] args )
	{
		new verifyResults(args).mainRun();
	}


	// initialization

	public verifyResults ( String[] args )
	{
		super( args );
	}


	// implementation of the CmdExecBase interface

	protected void execWorker( Model data, QueryMix queryMix, AbstractParameterPool parameterPool )
	{
		queryMix.setRun( 0 );
		while( queryMix.hasNext() )
		{
			Query query = queryMix.getNext();
			query.setParameters( parameterPool.getParametersForQuery(query) );
			System.out.println( "Executing query #" + String.valueOf(query.getNr()) + "." );
			executeQuery( query, data );
			queryMix.setCurrent( 0, -1.0 );
		}
		queryMix.finishRun();
	}


	// helpers

	protected void executeQuery ( Query q, Model data )
	{
		QueryExecution qe1 = QueryExecutionFactory.create( q.getQueryString(), data );
		QueryExecution qe2 = QueryExecutionFactory.create( q.getQueryString(), data );
		if ( q.getQueryType().equals(Query.SELECT_TYPE) )
		{
			ResultSetMem results1 = new ResultSetMem( qe1.execSelect() );
			ResultSetMem results2 = new ResultSetMem( qe2.execSelect() );
			if ( ! ResultSetCompare.equalsByValue(results1,results2) )
			{
				System.out.println( "The results for query #" + String.valueOf(q.getNr()) + " differ." );

// 				results1.reset();
// 				System.out.println( "results 1:" );
// 				ResultSetFormatter.out( System.out, results1 );
// 
// 				results2.reset();
// 				System.out.println( "results 2:" );
// 				ResultSetFormatter.out( System.out, results2 );
			}
		}
		else
		{
			Model result1 = null;
			Model result2 = null;
 			if ( q.getQueryType().equals(Query.CONSTRUCT_TYPE) )
			{
				result1 = qe1.execConstruct();
				result2 = qe2.execConstruct();
			}
			else if ( q.getQueryType().equals(Query.DESCRIBE_TYPE) )
			{
				result1 = qe1.execDescribe();
				result2 = qe2.execDescribe();
			}

			if ( ! result1.isIsomorphicWith(result2) )
			{
				System.out.println( "The results for query #" + String.valueOf(q.getNr()) + " differ." );
			}
		}
	}

}
